"""Initializes package mongodump-s3."""

from .s3 import S3
from .dump import MongoDump
from .notifications import Notifications

__version__ = '1.1.0'
__author__ = 'Vladislav I. Kulbatski'
__all__ = ['MongoDump', 'S3', 'Notifications', '__version__']



"""Main module contains functions needed by mongodump-s3 package."""

import os
import sys
import argparse
import logging

from pathlib import Path
from datetime import datetime
from dotenv import load_dotenv

from mongodump_s3 import S3
from mongodump_s3 import MongoDump
from mongodump_s3 import Notifications
from mongodump_s3 import __version__

logging.basicConfig(format='%(asctime)s - %(levelname)s - %(message)s',
                    level=logging.INFO)


class MongoDumpS3:
    """Implement CLI and handles command execution.

    Attributes:
        options: startup options passed to the application
    """

    def __init__(self):
        """Initializes MongoDumpS3 with startup options."""
        self.options = self._startup_options()
        self.exec()

    @staticmethod
    def _startup_options() -> argparse.Namespace:
        """Function implements CLI interface.

        Returns:
            argparse.Namespace: object that contains all passed flags
        """

        cli_parser = argparse.ArgumentParser(
            prog='mongodump-s3',
            usage='%(prog)s <options>',
            description='Export the content of a running server into .bson files'
            ' and uploads to provided S3 compatible storage.'
            ' By default loads required settings from environment'
            ' variables.',
            epilog='Email bug reports, questions, discussions'
            ' to mailto:hi@exesse.org.'
            ' Please star project on GitHub:'
            ' https://github.com/exesse/mongodump-s3',
            add_help=False)

        # Section: general options
        general = cli_parser.add_argument_group('general options')
        general.add_argument('-h',
                             '--help',
                             action='help',
                             default=argparse.SUPPRESS,
                             help='print usage')

        general.add_argument('-v',
                             '--version',
                             action='version',
                             version=f'%(prog)s {__version__}',
                             help='print the tool version and exit')

        # Section: output options
        output = cli_parser.add_argument_group('output options')
        output.add_argument(
            '-b',
            '--bucket',
            action='store',
            metavar='<S3 Bucket>',
            help='S3 bucket name for upload, defaults to \'mongodump\'')
        output.add_argument('-o',
                            '--out',
                            action='store',
                            metavar='<folder>',
                            help='output directory, defaults to \'dump\'')

        # Section: uri options
        uri = cli_parser.add_argument_group('uri options')
        uri.add_argument(
            '-u',
            '--uri',
            action='store',
            type=str,
            metavar='<uri>',
            help='mongodb uri connection string.'
            ' See official description here'
            ' https://docs.mongodb.com/manual/reference/connection-string')

        # Section: environmental options
        env = cli_parser.add_argument_group('environmental options')
        env.add_argument('-e',
                         '--env',
                         action='store',
                         type=str,
                         metavar='<env-file>',
                         help='path to file containing environmental variables')

        # Section: cloud storage options
        cloud_storage = cli_parser.add_argument_group('cloud storage options')
        cloud_storage.add_argument(
            '--azure',
            action='store',
            type=str,
            metavar='"<azure_storage_connection_string>"',
            help='connection string for storage account provided by Azure')

        cloud_storage.add_argument(
            '--aws',
            action='store',
            type=str,
            metavar='"<aws_access_key_id=value>'
            ' <aws_secret_access_key=value> <aws_region=value>"',
            help='AWS_ACCESS_KEY_ID, AWS_SECRET_ACCESS_KEY, AWS_REGION properties'
            ' provided by Amazon Web Services IAM. '
            'AWS_REGION defaults to \'us-west-2\' if not specified')
        cloud_storage.add_argument(
            '--gcp',
            action='store',
            type=str,
            metavar=
            '"<google_application_credentials=value> <google_region=value>"',
            help='path to service account file and optional Google Cloud Region.'
            ' GOOGLE_REGION defaults to \'us-multiregion\' if not specified')

        # Section: notification options
        notification = cli_parser.add_argument_group('notification options')
        notification.add_argument(
            '--email',
            action='store',
            metavar='<user@example.com>',
            help='email address which to notify upon the result')
        notification.add_argument(
            '--smtp',
            action='store',
            metavar='<mail-server.example.com>',
            help='SMTP relay server to use, defaults to \'localhost\'')
        notification.add_argument(
            '--telegram',
            action='store',
            type=str,
            metavar='"<telegram_token=value> <telegram_chat_id=value>"',
            help='Telegram API token and chat id to be used for notification. '
            ' See more: https://core.telegram.org/bots/api')

        return cli_parser.parse_args()

    @staticmethod
    def mask_env(scope: str = 'all') -> bool:
        """Masks environment variables when flags passed.

        Returns:
            True: if successful
            False: in case of failure
        """
        if scope == 'cloud':
            cloud_env_vars = [
                'AZURE_STORAGE_CONNECTION_STRING',
                'AWS_REGION',
                'AWS_ACCESS_KEY_ID',
                'AWS_SECRET_ACCESS_KEY',
                'GOOGLE_APPLICATION_CREDENTIALS',
                'GOOGLE_REGION',
            ]
        else:
            cloud_env_vars = [
                'MONGO_URI',
                'MONGO_OUTPUT_FOLDER',
                'MONGO_DUMP_BUCKET',
                'EMAIL',
                'SMTP_RELAY',
                'TELEGRAM_TOKEN',
                'TELEGRAM_CHAT_ID'
                'AZURE_STORAGE_CONNECTION_STRING',
                'AWS_REGION',
                'AWS_ACCESS_KEY_ID',
                'AWS_SECRET_ACCESS_KEY',
                'GOOGLE_APPLICATION_CREDENTIALS',
                'GOOGLE_REGION',
            ]
        try:
            for env in cloud_env_vars:
                os.unsetenv(env)
            return True
        except OSError as error_response:
            logging.error(error_response)
        return False

    @staticmethod
    def _str_to_dict(argument_string: str) -> dict:
        """Converts passed long key-value argument to python dictionary.

        Returns:
            result: dict, containing passed options
        """
        result = {}
        separate_kv = argument_string.split(' ')
        for flags in separate_kv:
            flag_key, flag_value = flags.split('=')
            result[flag_key] = flag_value
        return result

    @staticmethod
    def _set_env(env_kwargs: dict) -> bool:
        """Sets environment variables from passed dictionary.

        Returns:
            True: if successful
            False: in case of failure
        """
        try:
            for key in env_kwargs:
                os.environ[str(key).upper()] = str(env_kwargs[key])
        except OSError as error_response:
            logging.error(error_response)
            logging.error('Application was not able to set env variables.'
                          ' Please report the bug to mailto:hi@exesse.org')
            return False
        return True

    @staticmethod
    def debug_env() -> None:
        """Helper method that prints all possible env vars."""
        logging.debug('MONGO_URI set to "%s"', os.getenv('MONGO_URI'))
        logging.debug('MONGO_OUTPUT_FOLDER set to "%s"',
                      os.getenv('MONGO_OUTPUT_FOLDER'))
        logging.debug('MONGO_DUMP_BUCKET set to "%s"',
                      os.getenv('MONGO_DUMP_BUCKET'))
        logging.debug('EMAIL set to "%s"', os.getenv('EMAIL'))
        logging.debug('SMTP_RELAY set to "%s"', os.getenv('SMTP_RELAY'))
        logging.debug('TELEGRAM_TOKEN set to "%s"', os.getenv('TELEGRAM_TOKEN'))
        logging.debug('TELEGRAM_CHAT_ID set to "%s"',
                      os.getenv('TELEGRAM_CHAT_ID'))
        logging.debug('AZURE_STORAGE_CONNECTION_STRING set to "%s"',
                      os.getenv('AZURE_STORAGE_CONNECTION_STRING'))
        logging.debug('AWS_REGION set to "%s"', os.getenv('AWS_REGION'))
        logging.debug('AWS_ACCESS_KEY_ID set to "%s"',
                      os.getenv('AWS_ACCESS_KEY_ID'))
        logging.debug('AWS_SECRET_ACCESS_KEY set to "%s"',
                      os.getenv('AWS_SECRET_ACCESS_KEY'))
        logging.debug('GOOGLE_APPLICATION_CREDENTIALS set to "%s"',
                      os.getenv('GOOGLE_APPLICATION_CREDENTIALS'))
        logging.debug('GOOGLE_REGION set to "%s"', os.getenv('GOOGLE_REGION'))

    def _check_env_file(self) -> bool:
        """Checks if env-file was passed and clears env if True.

        Returns:
            True: if successful
            False: in case of failure
        """
        if self.options.env:
            env_file_path = Path(self.options.env)
            if env_file_path.is_file():
                load_dotenv(dotenv_path=env_file_path)
                self.debug_env()
                return True
            logging.error(
                'Provided env file "%s" does not exists. Please check.',
                self.options.env)

        return False

    def _check_output_flags(self) -> bool:
        """Checks for passed output related startup flags.

        Returns:
            True: if successful
            False: in case of failure
        """
        if self.options.bucket or self.options.out:
            bucket = {'MONGO_DUMP_BUCKET': self.options.bucket}
            out = {'MONGO_OUTPUT_FOLDER': self.options.out}

            if not self._set_env(bucket):
                logging.error('Failed to set variable "%s"', bucket)
                return False
            if not self._set_env(out):
                logging.error('Failed to set variable "%s"', out)
                return False

            return True

        return False

    def _check_cloud_flags(self) -> bool:
        """Checks for S3 cloud specific flags.

        Returns:
            True: if successful
            False: in case of failure
        """
        if self.options.azure or self.options.aws or self.options.gcp:
            if self.mask_env('cloud'):

                if self.options.azure:
                    azure = {
                        'AZURE_STORAGE_CONNECTION_STRING': self.options.azure
                    }
                    if not self._set_env(azure):
                        logging.error('Failed to set variable "%s"', azure)
                        return False
                if self.options.gcp:
                    gcp = self._str_to_dict(self.options.gcp)
                    if not self._set_env(gcp):
                        logging.error('Failed to set variable "%s"', gcp)
                        return False
                if self.options.aws:
                    aws = self._str_to_dict(self.options.aws)
                    if not self._set_env(aws):
                        logging.error('Failed to set variable "%s"', aws)
                        return False

                return True

        return False

    def _check_notifications_flags(self) -> bool:
        """Checks for passed notification related startup flags.

        Returns:
            True: if successful
            False: in case of failure
        """
        if self.options.email or self.options.smtp or self.options.telegram:
            email = {'EMAIL': self.options.email}
            smtp = {'SMTP_RELAY': self.options.smtp}
            telegram = self._str_to_dict(self.options.telegram)

            if not self._set_env(email):
                logging.error('Failed to set variable "%s"', email)
                return False
            if not self._set_env(smtp):
                logging.error('Failed to set variable "%s"', smtp)
                return False
            if not self._set_env(telegram):
                logging.error('Failed to set variable "%s"', telegram)
                return False

            return True

        return False

    def _prepare_app_env(self) -> bool:
        """Prepares working environment.

        Returns:
            True: if successful
            False: in case of failure
        """
        if self._check_env_file():
            logging.info('App properties were provided with env file.')
            return True
        if self.options.uri:
            uri = {'MONGO_URI': self.options.uri}
            logging.info('MongoDB uri was passed as startup flag.')
            if not self._set_env(uri):
                return False
        if self._check_output_flags():
            logging.info('Output properties were provided with startup flags.'
                         ' Masking related env vars now.')
        if self._check_cloud_flags():
            logging.info(
                'S3 connection properties were provided with startup flags.'
                ' Masking related env vars now.')
        if self._check_notifications_flags():
            logging.info(
                'Notification properties were provided with startup flags.'
                ' Masking related env vars now.')
        self.debug_env()
        return True

    def exec(self) -> bool:
        """Helper function that executes class MongoDUmpS3.

        Returns:
            True: if successful
            False: in case of failure
        """
        start = datetime.now()
        if self._prepare_app_env():
            failure = '\U0001F4A9 mongodump-s3 failed. Please see logs.'
            mongodump = MongoDump()
            dump_result = mongodump.exec()

            if dump_result['dump'] == 'False':
                Notifications(failure)
                mongodump.cleanup()
                return False

            dump_size = dump_result['size']
            dump_path = dump_result['path']
            dump_path_parent = str(Path(dump_path).parent)

            s3_upload = S3()
            s3_upload_result = s3_upload.upload_local_folder(
                dump_path, dump_path_parent)

            if s3_upload_result:
                end = str(datetime.now() - start)[:-7]
                success = '\U0001F4A5 mongodump-s3 finished the job.' \
                          '\n\U0001F9BA Dump size is %s.' \
                          '\n\U0001F312 Processing time is %s' % (dump_size, end)
                Notifications(success)
                mongodump.cleanup()
                return True
            mongodump.cleanup()
        return False


def main():
    """Wraps complete package to end-user."""
    run_app = MongoDumpS3()
    if run_app:
        sys.exit(0)
    sys.exit(1)


if __name__ == '__main__':
    main()



"""Module contains helper methods used by other modules in the package."""


def env_exists(env_variable: str) -> bool:
    """Validates if env variable was provided and is not an empty string.

    Args:
        env_variable: str, name of the env variable

    Returns:
        True: if env provide and not an empty string
        False: if env not provided or an empty string
    """
    if env_variable is not None and env_variable != '':
        return True
    return False



"""Module contains cloud S3 storage related methods."""

import os
import logging

from typing import Any
from pathlib import Path
from azure.storage.blob import BlobServiceClient as AzureClient
from azure.core.exceptions import AzureError
from boto3 import client as aws_client
from botocore.exceptions import ClientError as AmazonClientError
from google.cloud import storage as gcp_client
from google.cloud.exceptions import ClientError as GoogleClientError
from google.auth.exceptions import DefaultCredentialsError as GoogleAuthError

from .helpers import env_exists


class S3:
    """Implements S3 storage related operations.

    Attributes:
        s3_bucket: name of the S3 compatible bucket container
        providers: dict, contains S3 clients object generated from provided env parameters
        create_bucket: bool, parameter specifies if non-existing buckets should be created
    """

    def __init__(self, create_buckets: bool = True):
        """Initializes class S3 with bucket name."""
        self.create_bucket = create_buckets
        s3_bucket = os.getenv('MONGO_DUMP_BUCKET')
        if not env_exists(s3_bucket):
            s3_bucket = 'mongodump'
        self.s3_bucket = s3_bucket
        self.providers = self.create_storage_clients()

    def _make_azure_client(self) -> Any:
        """Creates Azure client with provided credentials.

        Returns:
            False: if no connection string provided
            azure_s3: bucket object that will be used to operate with Azure API
        """
        azure_connection_string = os.getenv('AZURE_STORAGE_CONNECTION_STRING')

        if env_exists(azure_connection_string):
            logging.info('Azure connection parameters found.')
            try:
                azure_s3 = AzureClient.from_connection_string(
                    azure_connection_string)
                azure_buckets = [
                    container.name for container in azure_s3.list_containers()
                ]
            except ValueError as error_response:
                logging.error(error_response)
                return False
            if self.s3_bucket in azure_buckets:
                logging.info(
                    'Container "%s" already exists on Azure and owned by you.',
                    self.s3_bucket)
                return azure_s3
            if self.create_bucket:
                try:
                    azure_s3.create_container(self.s3_bucket)
                    logging.info(
                        'Container "%s" successfully created on Azure.',
                        self.s3_bucket)
                    return azure_s3
                except AzureError as error_response:
                    logging.error(error_response)
        return False

    def _make_aws_client(self) -> Any:
        """Creates AWS boto3 client with provided credentials.

        Returns:
            False: if no connection string provided
            aws_s3: bucket object that will be used to operate with Amazon API
        """
        aws_access_key_id = os.getenv('AWS_ACCESS_KEY_ID')
        aws_secret_access_key = os.getenv('AWS_SECRET_ACCESS_KEY')
        aws_region = os.getenv('AWS_REGION')

        if env_exists(aws_access_key_id) and env_exists(aws_secret_access_key):
            logging.info('AWS connection parameters found.')
            try:
                aws_s3 = aws_client('s3')
                aws_buckets = [
                    bucket['Name']
                    for bucket in (aws_s3.list_buckets())['Buckets']
                ]
            except AmazonClientError as error_response:
                logging.error(error_response)
                return False
            if self.s3_bucket in aws_buckets:
                logging.info(
                    'Container "%s" already exists on AWS and owned by you.',
                    self.s3_bucket)
                return aws_s3
            if not env_exists(aws_region):
                aws_region = 'us-west-2'
            aws_s3 = aws_client('s3', region_name=aws_region)
            location = {'LocationConstraint': aws_region}
            try:
                aws_s3.create_bucket(Bucket=self.s3_bucket,
                                     CreateBucketConfiguration=location)
                logging.info('"%s" was created in region "%s" on AWS',
                             self.s3_bucket, aws_region)
                return aws_s3
            except AmazonClientError as error_response:
                logging.error(error_response)
        return False

    def _make_gcp_client(self) -> Any:
        """Creates GCP client with provided credentials.

        Returns:
            False: if no connection string provided
            gcp_s3: bucket object that will be used to operate with GCP API
        """
        google_application_credentials = os.getenv(
            'GOOGLE_APPLICATION_CREDENTIALS')
        google_region = os.getenv('GOOGLE_REGION')

        if env_exists(google_application_credentials) and Path(
                google_application_credentials).is_file():
            logging.info('GCP connection parameters found.')
            try:
                gcp_s3 = gcp_client.Client()
                gcp_buckets = [bucket.name for bucket in gcp_s3.list_buckets()]
            except GoogleAuthError as error_response:
                logging.error(error_response)
                return False
            if self.s3_bucket in gcp_buckets:
                logging.info(
                    'Container "%s" already exists on GCP and owned by you.',
                    self.s3_bucket)
                return gcp_s3
            if not google_region:
                google_region = 'us'
            try:
                gcp_s3.create_bucket(self.s3_bucket, location=google_region)
                logging.info('"%s" was created in region "%s" on GCP',
                             self.s3_bucket, google_region)
                return gcp_s3
            except GoogleClientError as error_response:
                logging.error(error_response)
        return False

    def create_storage_clients(self) -> dict:
        """Creates client connections from env for the cloud to be used.

        Returns:
            providers: dict, containing provider specific s3-storage clients
        """
        providers = {}
        azure_s3 = self._make_azure_client()
        aws_s3 = self._make_aws_client()
        gcp_s3 = self._make_gcp_client()
        if azure_s3:
            providers['azure'] = azure_s3
        if aws_s3:
            providers['aws'] = aws_s3
        if gcp_s3:
            providers['gcp'] = gcp_s3
        logging.debug(providers)
        return providers

    def upload_local_file(self, local_file_path: str,
                          *remote_file_path: str) -> bool:
        """Uploads the file to the specified bucket.

        Args:
            local_file_path: str, full path to local file
            remote_file_path: str, optional name for remote file,
                              if not provided will be the same as local_file_path

        Returns:
            False: if folder upload failed, or folder doesn't exists
            True: if folder upload succeeded
        """
        if not remote_file_path:
            remote_file_path = local_file_path
        remote_file_path = str(remote_file_path[0])

        if Path(local_file_path).is_file():
            if 'azure' in self.providers:
                azure_s3 = self.providers['azure']
                try:
                    blob = azure_s3.get_blob_client(container=self.s3_bucket,
                                                    blob=remote_file_path)
                    with open(local_file_path, "rb") as data:
                        blob.upload_blob(data)
                    logging.info(
                        '"%s" was successfully uploaded to Azure '
                        'and stored as "%s" on "%s" bucket.', local_file_path,
                        remote_file_path, self.s3_bucket)
                except AzureError as error_response:
                    exists_error = 'The specified blob already exists.'
                    if exists_error in str(error_response):
                        logging.info(
                            '"%s" already exists in the "%s" bucket on Azure.',
                            remote_file_path, self.s3_bucket)
                    else:
                        logging.error(error_response)
            if 'aws' in self.providers:
                aws_s3 = self.providers['aws']
                try:
                    aws_s3.upload_file(local_file_path, self.s3_bucket,
                                       remote_file_path)
                    logging.info(
                        '"%s" was successfully uploaded to AWS '
                        'and stored as "%s" on "%s" bucket.', local_file_path,
                        remote_file_path, self.s3_bucket)
                except AmazonClientError as error_response:
                    logging.error(error_response)
            if 'gcp' in self.providers:
                gcp_s3 = self.providers['gcp']
                try:
                    gcp_s3_client = gcp_s3.get_bucket(self.s3_bucket)
                    blob = gcp_s3_client.blob(remote_file_path)
                    blob.upload_from_filename(local_file_path)
                    logging.info(
                        '"%s" was successfully uploaded to GCP '
                        'and stored as "%s" on "%s" bucket.', local_file_path,
                        remote_file_path, self.s3_bucket)
                except GoogleClientError as error_response:
                    logging.error(error_response)
            return True
        logging.error('"%s" does not exists', local_file_path)
        return False

    def upload_local_folder(self,
                            local_folder_path: str,
                            remove_path_parent: str = '') -> bool:
        """Uploads local folder to remote S3 storage.

        Args:
            local_folder_path: str, full path to the local folder to be uploaded
            remove_path_parent: str, optional, part that will be subtracted from
                                full file path in the naming of remote file

        Returns:
            False: if folder upload failed, or folder doesn't exists
            True: if folder upload succeeded
        """
        if Path(local_folder_path).is_dir():
            for local_file in Path(local_folder_path).rglob('*'):
                if Path(local_file).is_file():
                    if remove_path_parent == '':
                        remote_file = local_file
                    else:
                        remote_file = local_file.relative_to(remove_path_parent)
                    local_file_name = str(local_file)
                    remote_file_name = str(remote_file)
                    self.upload_local_file(local_file_name, remote_file_name)
            logging.info('"%s" was successfully uploaded to s3 storage',
                         local_folder_path)
            return True
        logging.error('"%s" folder does not exists. Please check.',
                      local_folder_path)
        return False



"""Module implements methods needed to perform dump related operations."""

import os
import sys
import shlex
import socket
import logging
import datetime
import shutil

from subprocess import Popen, PIPE
from pathlib import Path
from typing import Union
from hurry.filesize import size, si

from .helpers import env_exists


class MongoDump:
    """Dumps Mongo database and prepares bson files for the transfer.

    Attributes:
        mongo_uri: connection string in format

        "mongodb://user:password@server:[port]/?replicaSet=NAME&authSource=admin"

        output_folder: path to where local dump will be performed
    """

    def __init__(self):
        """Initializes MongoDump with connection URI."""
        self.mongo_uri = os.getenv('MONGO_URI')

        if not env_exists(self.mongo_uri):
            logging.error(
                'No MongoDB connection URI provided. Nothing to do - exiting now.'
            )
            sys.exit(1)

        mongo_output_folder = os.getenv('MONGO_OUTPUT_FOLDER')

        if not env_exists(mongo_output_folder):
            mongo_output_folder = 'dump'
        self.output_folder = Path.absolute(Path(mongo_output_folder))

    def _strip_mongo_uri(self) -> list:
        """Strips mongo_uri to get list of mongodb servers provided.

        Returns:
            mongo_srv_list: list, contains tuples in a way (mongo_host:str, *mongo_port:str)
        """
        try:
            mongo_srv_string = (str(self.mongo_uri).split('/'))[2]
        except IndexError:
            mongo_srv_string = str(self.mongo_uri)
        try:
            mongo_srv_string_no_creds = (mongo_srv_string.split('@'))[1]
        except IndexError:
            mongo_srv_string_no_creds = mongo_srv_string
        mongo_srv_list = [
            tuple(location.split(':'))
            for location in mongo_srv_string_no_creds.split(',')
        ]
        return mongo_srv_list

    def _check_mongo_socket(self) -> bool:
        """Checks if at least one of given servers in mongo_uri is live.

        Returns:
            True: for the first alive server
            False: if none of given servers are live
        """
        for mongo_srv in self._strip_mongo_uri():
            mongo_host = str(mongo_srv[0])
            mongo_port = 27017
            if len(mongo_srv) == '2':
                mongo_port = (int(mongo_srv[1]))
            location = (mongo_host, mongo_port)
            with socket.socket(socket.AF_INET,
                               socket.SOCK_STREAM) as mongo_socket:
                try:
                    connection_state = mongo_socket.connect_ex(location)
                    if connection_state == 0:
                        return True
                except socket.gaierror:
                    logging.error('MongoDB is not serving at "%s" port "%s".',
                                 mongo_host, str(mongo_port))
        return False

    def _get_dump_size(self) -> Union[str, bool]:
        """Calculates the size for the dump folder and returns it in SI value.

        Returns:
            folder_size: str, folder size in SI
            'False' in case of failure
        """
        try:
            summary = sum(f.stat().st_size
                          for f in Path(self.output_folder).rglob('*')
                          if f.is_file())
        except os.error:
            return False
        folder_size = size(summary, system=si)
        logging.info('Dump folder is stored at "%s" and "%s" large.',
                     self.output_folder, folder_size)
        return folder_size

    def _rename_dump(self) -> Union[str, bool]:
        """Renames temporary dump folder by adding current date.

        Returns:
            folder_name: str, new folder name
        """
        dump_folder_name = f'{str(self.output_folder)}-{str(datetime.date.today())}'
        if Path(dump_folder_name).is_dir():
            logging.warning('Dump folder "%s" already exists. Removing now.',
                            dump_folder_name)
            shutil.rmtree(dump_folder_name)
        try:
            shutil.move(str(self.output_folder), dump_folder_name)
            logging.info('Dump folder renamed and stored at "%s"',
                         dump_folder_name)
        except os.error as error_msg:
            logging.error(error_msg)
            return False
        return dump_folder_name

    def dump_db(self) -> bool:
        """Performs dump of the database to the local folder.

        Returns:
            The result of dump command.
            True: if successful
            False: in case of failure
        """
        if self._check_mongo_socket():
            cmd = f'mongodump --uri="{self.mongo_uri}" --gzip --out {self.output_folder}'
            args = shlex.split(cmd)
            dump_process = Popen(args, stdout=PIPE)
            with dump_process.stdout:
                for line in iter(dump_process.stdout.readline, b''):
                    dump_process_output = bytes(line).decode('utf-8').strip(
                        '\n')
                    logging.info(dump_process_output)
            exit_code = dump_process.wait()
            if exit_code == 0:
                return True
        return False

    def cleanup(self) -> bool:
        """Performs cleanup steps on exist or failure.

        Returns:
            True: if successful
            False: in case of failure
        """
        try:
            dump_folder_name = f'{str(self.output_folder)}-{str(datetime.date.today())}'
            if Path(dump_folder_name).is_dir():
                logging.warning(
                    'Performing cleanup steps - "%s" removed.',
                    dump_folder_name)
                shutil.rmtree(dump_folder_name)
            if Path(str(self.output_folder)).is_dir():
                logging.warning(
                    'Performing cleanup steps - "%s" removed.',
                    str(self.output_folder))
                shutil.rmtree(dump_folder_name)
            return True
        except OSError:
            logging.error('Application failed to perform cleanup steps')
            return False

    def exec(self) -> dict:
        """Wraps all class methods for single start

        Returns:
            result: dict, result of dump operations as dictionary
        """
        dump_state = self.dump_db()
        if dump_state:
            result = {
                'dump': str(dump_state),
                'size': self._get_dump_size(),
                'path': self._rename_dump()
            }
        else:
            result = {'dump': str(dump_state)}
        logging.debug(str(result))
        return result



"""Module implements various notification methods."""

import os
import smtplib
import logging

from socket import gaierror
from email.message import EmailMessage
from requests import post

from .helpers import env_exists


class Notifications:
    """Handles notifications requests.

    Attributes:
        message: str, plain text msg that will be send
    """

    def __init__(self, message: str) -> None:
        """Initializes class Notifications.

        Args:
            message: str, plain text body that will be sent
        """
        self.message = message
        self.send_email_notification()
        self.send_telegram_notification()

    def send_telegram_notification(self) -> bool:
        """"Handler for Telegram notifications.

        Returns:
            Fail: if no env variables were provided
            True: if environment values exists and msg was sent
        """
        telegram_chat_id = os.getenv('TELEGRAM_CHAT_ID')
        telegram_token = os.getenv('TELEGRAM_TOKEN')

        if env_exists(telegram_chat_id) and env_exists(telegram_token):
            telegram_send_message = f'https://api.telegram.org/bot{telegram_token}/sendMessage'
            text_body = {'chat_id': telegram_chat_id, 'text': self.message}
            try:
                post(telegram_send_message, text_body).json()
                logging.info('Telegram notification sent to "%s".',
                             telegram_chat_id)
                return True
            except ConnectionError as error_response:
                details = str(error_response).split(':')[0]
                logging.error(details)
        return False

    def send_email_notification(self) -> bool:
        """Handler for Email notifications.

        Returns:
            Fail: if no env variables were provided
            True: if environment values exists and msg was sent
        """
        email = os.getenv('EMAIL')
        smtp_relay = os.getenv('SMTP_RELAY')

        if env_exists(email):
            if not env_exists(smtp_relay):
                smtp_relay = 'localhost'
            try:
                msg = EmailMessage()
                msg['Subject'] = '\U0001F4D1 [mongo-dump] status report'
                msg['From'] = 'mongo-dump@service.io'
                msg['To'] = email
                msg.set_content(self.message)
                with smtplib.SMTP(smtp_relay) as smtp:
                    smtp.send_message(msg)
                logging.info('Email was sent to "%s" via smtp relay "%s".',
                             email, smtp_relay)
                return True
            except gaierror:
                logging.error(
                    'smtp relay server "%s" is not available. Please check.',
                    smtp_relay)
            except OSError:
                logging.error(
                    'smtp relay server name "%s" could not be resolved over DNS. Please check.',
                    smtp_relay)
        return False



import pathlib
from setuptools import setup

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

# This call to setup() does all the work
setup(
    name="mongodump-s3",
    version="1.1.1",
    description="Backup utility for MongoDB. "
                "Compatible with Azure, Amazon Web Services and Google Cloud Platform.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/exesse/mongodump-s3",
    author="Vladislav I. Kulbatski ",
    author_email="hi@exesse.org",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=["mongodump_s3"],
    include_package_data=False,
    install_requires=[
        "requests>=2.26.0",
        "hurry.filesize==0.9",
        "python-dotenv>=0.18.0",
        "azure-storage-blob>=12.8.1",
        "boto3>=1.17.111",
        "google-cloud-storage>=1.41.0"
        ],
    entry_points={
        "console_scripts": [
            "mongodump-s3=mongodump_s3.__main__:main",
        ]
    },
)
